"""Console script for brellium_llms."""
# pylint: disable=import-error, too-many-arguments

import sys
import click

from anthropic import Client
from dotenv import load_dotenv
from langchain.chat_models import ChatAnthropic, ChatOpenAI

from constants import QUERIES

from map_reduce import create_map_reduce_medical_document_retrieval_qa_chain
from map_refine import create_map_refine_medical_document_retrieval_qa_chain
import pandas as pd
from lib import (
    create_local_qdrant_vectorstore,
    create_sagemaker_llm,
    MedicalDocumentRetriever,
    print_inference_result,
)
from enum import Enum
load_dotenv()  # take environment variables from .env.
Chains = Enum('Chains', ['MAP_REDUCE_GPT', 'MAP_REDUCE_MPT', 'MAP_REFINE_MPT', 'MAP_REFINE_ANTHROPIC', 'MAP_REDUCE_ANTHROPIC'])

anthropic_llm = ChatAnthropic(
    client=Client(
        "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"
    ),
    temperature=1e-9,
)

mpt_llm = create_sagemaker_llm(
    endpoint_name="jumpstart-dft-hf1-textgeneration1-mpt-7b-instruct-bf16",
    model_kwargs={
        "temperature": 1e-9,
        "max_new_tokens": 256,
    },
)

open_ai_llm = ChatOpenAI(
    temperature=1e-9,
)

gpt_4_llm = ChatOpenAI(
    temperature=1e-9,
    model="gpt-4",
)

qdrant_db = create_local_qdrant_vectorstore(
    file_path="./test-input.txt",
)


def run_mpt_map_reduce():
    """Run the MPT LLM on the medical document retrieval QA chain."""
    map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
        ),
        map_llm=mpt_llm,
        reduce_llm=open_ai_llm,
        return_map_steps=True,
    )

    for question_idx, prompt_obj in enumerate(QUERIES):
        prompt_cur = prompt_obj.get("query")
        qa_response = map_reduce_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3000,
                "k": 4,
            }
        )
        print_inference_result(question_idx + 1, qa_response)


def run_mpt_map_refine():
    """Run the MPT LLM on the medical document retrieval QA chain."""
    map_refine_retrieval_qa = create_map_refine_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
        ),
        map_llm=mpt_llm,
        refine_llm=open_ai_llm,
        return_refine_steps=True,
    )

    for question_idx, prompt_obj in enumerate(QUERIES):
        prompt_cur = prompt_obj.get("query")
        qa_response = map_refine_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3000,
                "k": 4,
            }
        )
        print_inference_result(question_idx + 1, qa_response)


def run_anthropic_map_reduce():
    """Run the Anthropic LLM on the medical document retrieval QA chain."""

    map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=anthropic_llm,
        reduce_llm=anthropic_llm,
        return_map_steps=True,
    )

    for question_idx, prompt_obj in enumerate(QUERIES):
        prompt_cur = prompt_obj.get("query")
        qa_response = map_reduce_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3_000,
                "k": 4,
            }
        )
        print_inference_result(question_idx + 1, qa_response)


def run_anthropic_map_refine():
    """Run the Anthropic LLM on the medical document retrieval QA chain."""

    map_refine_retrieval_qa = create_map_refine_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=anthropic_llm,
        refine_llm=anthropic_llm,
        return_refine_steps=True,
    )

    for question_idx, prompt_obj in enumerate(QUERIES):
        prompt_cur = prompt_obj.get("query")
        qa_response = map_refine_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 75_000,
                "k": 6,
            }
        )
        print_inference_result(question_idx + 1, qa_response)


def run_gpt_map_reduce():
    """Run the GPT LLM on the medical document retrieval QA chain."""
    map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=open_ai_llm,
        reduce_llm=gpt_4_llm,
        return_map_steps=False,
    )

    for question_idx, prompt_obj in enumerate(QUERIES):
        prompt_cur = prompt_obj.get("query")
        qa_response = map_reduce_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3000,
                "k": 8,
            }
        )
        print_inference_result(question_idx + 1, qa_response)

def run_question(query, chain_type):
    print("Running question: ", query)
    if chain_type == Chains.MAP_REDUCE_GPT:
        print("Running MAP_REDUCE_GPT")
        map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=open_ai_llm,
        reduce_llm=gpt_4_llm,
        return_map_steps=False,
        )
        qa_response = map_reduce_retrieval_qa(
            inputs={
                "query": query,
                "token_max": 3000,
                "k": 8,
            }
        )
        print_inference_result(0, qa_response)
    elif chain_type == Chains.MAP_REDUCE_MPT:
        print("Running MAP_REDUCE_MPT")
        map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
        ),
        map_llm=mpt_llm,
        reduce_llm=open_ai_llm,
        return_map_steps=True,
        )
        qa_response = map_reduce_retrieval_qa(
            inputs={
                "query": query,
                "token_max": 3000,
                "k": 4,
            }
        )
        print_inference_result(0, qa_response)
    elif chain_type == Chains.MAP_REFINE_MPT:
        print("Running MAP_REFINE_MPT")
        map_refine_retrieval_qa = create_map_refine_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
        ),
        map_llm=mpt_llm,
        refine_llm=open_ai_llm,
        return_refine_steps=True,
        )
        qa_response = map_refine_retrieval_qa(
            inputs={
                "query": query,
                "token_max": 3000,
                "k": 4,
            }
        )
        print_inference_result(0, qa_response)
    elif chain_type == Chains.MAP_REDUCE_ANTHROPIC:
        print("Running MAP_REDUCE_ANTHROPIC")
        map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=anthropic_llm,
        reduce_llm=anthropic_llm,
        return_map_steps=True,
        )
        qa_response = map_reduce_retrieval_qa(
            inputs={
                "query": query,
                "token_max": 3_000,
                "k": 4,
            }
        )
        print_inference_result(0, qa_response)
    elif chain_type == Chains.MAP_REFINE_ANTHROPIC:
        print("Running MAP_REFINE_ANTHROPIC")
        map_refine_retrieval_qa = create_map_refine_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=anthropic_llm,
        refine_llm=anthropic_llm,
        return_refine_steps=True,
        )
        qa_response = map_refine_retrieval_qa(
            inputs={
                "query": query,
                "token_max": 75_000,
                "k": 6,
            }
        )
        print_inference_result(0, qa_response)

def run_all_models():
    gpt_map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=open_ai_llm,
        reduce_llm=gpt_4_llm,
        return_map_steps=False,
    )
    mpt_map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
        ),
        map_llm=mpt_llm,
        reduce_llm=open_ai_llm,
        return_map_steps=True,
    )
    mpt_map_refine_retrieval_qa = create_map_refine_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
        ),
        map_llm=mpt_llm,
        refine_llm=open_ai_llm,
        return_refine_steps=True,
    )
    anthropic_map_reduce_retrieval_qa = create_map_reduce_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=anthropic_llm,
        reduce_llm=anthropic_llm,
        return_map_steps=True,
    )
    anthropic_map_refine_retrieval_qa = create_map_refine_medical_document_retrieval_qa_chain(
        retriever=MedicalDocumentRetriever(
            vectorstore=qdrant_db,
            queries=QUERIES,
            default_k=4,
        ),
        map_llm=anthropic_llm,
        refine_llm=anthropic_llm,
        return_refine_steps=True,
    )
    df = pd.DataFrame(columns=["query", "gpt_map_reduce", "mpt_map_reduce", "mpt_map_refine", "anthropic_map_reduce", "anthropic_map_refine"])
    for question_idx, prompt_obj in enumerate(QUERIES):
        print(f"Running question {question_idx}")
        prompt_cur = prompt_obj.get("query")
        gpt_map_reduce_qa_response = gpt_map_reduce_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3000,
                "k": 6,
            }
        )
        mpt_map_reduce_qa_response = mpt_map_reduce_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3000,
                "k": 4,
            }
        )
        mpt_map_refine_qa_response = mpt_map_refine_retrieval_qa(
            inputs={
                "query": prompt_cur,
                "token_max": 3000,
                "k": 4,
            }
        )
        # anthropic_map_reduce_qa_response = anthropic_map_reduce_retrieval_qa(
        #     inputs={
        #         "query": prompt_cur,
        #         "token_max": 3_000,
        #         "k": 4,
        #     }
        # )
        # anthropic_map_refine_qa_response = anthropic_map_refine_retrieval_qa(
        #     inputs={
        #         "query": prompt_cur,
        #         "token_max": 75_000,
        #         "k": 6,
        #     }
        # )
        print(mpt_map_reduce_qa_response.get('result'))
        print(mpt_map_refine_qa_response.get('result'))
        anthropic_map_refine_qa_response = { "result": "N/A" }
        anthropic_map_reduce_qa_response = { "result": "N/A" }
        df.loc[question_idx] = [prompt_cur, gpt_map_reduce_qa_response.get('result'), mpt_map_reduce_qa_response.get('result'), mpt_map_refine_qa_response.get('result'), anthropic_map_reduce_qa_response.get('result'), anthropic_map_refine_qa_response.get('result')]
    df.to_csv("results.csv")
@click.command()
def main(args=None):
    """Console script for brellium_llms."""
    # click.echo("Replace this message by putting your code into "
    #            "brellium_llms.cli.main")
    # click.echo("See click documentation at https://click.palletsprojects.com/")

    # run_anthropic_map_reduce()
    # run_anthropic_map_refine()
    # run_mpt_map_refine()

    # run_mpt_map_reduce()

    # run_gpt_map_reduce()
    run_all_models()
    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
