"""main.py"""

from langchain.document_loaders import TextLoader
from langchain.embeddings import OpenAIEmbeddings
from langchain.embeddings.base import Embeddings
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain.vectorstores import Qdrant
from qdrant_client.models import Distance, HnswConfigDiff
from tiktoken import encoding_for_model


def num_tokens(text: str) -> int:
    """Return the number of tokens in a string."""
    encoding = encoding_for_model("gpt-4")
    return len(encoding.encode(text))


def create_local_qdrant_vectorstore(
    file_path: str,
    embeddings: Embeddings = OpenAIEmbeddings(
        chunk_size=1,
    ),
    text_splitter_chunk_size=512,
    collection_name="medical_document",
):
    """create an in-memory qdrant vectorstore from a single medical document file"""

    loader = TextLoader(file_path=file_path)
    documents = loader.load()

    text_splitter = RecursiveCharacterTextSplitter(
        chunk_size=text_splitter_chunk_size,
        chunk_overlap=0,
        length_function=num_tokens,
    )

    documents = text_splitter.split_documents(documents=documents)

    qdrant_db = Qdrant.from_documents(
        documents=documents,
        embedding=embeddings,
        location=":memory:",
        collection_name=collection_name,
        hnsw_config=HnswConfigDiff(
            payload_m=16,
            m=0,
            ef_construct=100,
            full_scan_threshold=1000,
        ),
        distance_func=Distance.COSINE,
    )

    return qdrant_db
