"""medical_document_retrieval_qa_chain.py"""

import time

from typing import Any, Dict, List, Optional
from pydantic import Field

from langchain.callbacks.manager import CallbackManagerForChainRun
from langchain.chains import RetrievalQA
from langchain.schema import Document

from .medical_document_retriever import MedicalDocumentRetriever


class MedicalDocumentRetrievalQA(RetrievalQA):
    """Chain for question-answering against an index."""

    retriever: MedicalDocumentRetriever = Field(exclude=True)

    # pylint: disable=arguments-differ
    def _get_docs(self, question: str, k: int) -> List[Document]:
        return self.retriever.get_relevant_documents(question, k=k)

    # pylint: disable=arguments-differ
    async def _aget_docs(self, question: str, k: int) -> List[Document]:
        return await self.retriever.aget_relevant_documents(question, k=k)

    def _call(
        self,
        inputs: Dict[str, Any],
        run_manager: Optional[CallbackManagerForChainRun] = None,
    ) -> Dict[str, Any]:
        """Run get_relevant_text and llm on input query."""
        start = time.time()

        _run_manager = run_manager or CallbackManagerForChainRun.get_noop_manager()

        question = inputs[self.input_key]

        token_max = inputs.get("token_max", 3000)

        k = inputs.get("k", self.retriever.default_k)

        docs = self._get_docs(question, k)

        res = self.combine_documents_chain(
            inputs={
                "input_documents": docs,
                "question": question,
                "token_max": token_max,
            },
            callbacks=_run_manager.get_child(),
        )

        end = time.time()

        results = {
            "runtime": end - start,
            self.output_key: res.get(self.combine_documents_chain.output_key),
        }

        if res.get("intermediate_steps") is not None:
            results["intermediate_steps"] = res["intermediate_steps"]

        if self.return_source_documents:
            results["source_documents"] = docs

        return results

    @property
    def _chain_type(self) -> str:
        """Return the chain type."""
        return "medical_document_retrieval_qa"
